#
# Aura Engine (https://gitlab.servus.at/aura/engine)
#
# Copyright (C) 2020 - The Aura Engine Team.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import json
import logging
import requests

from modules.base.utils import SimpleUtil as SU



class TrackserviceHandler():
    """
    Sends the trackservice entry to the `engine-api` REST endpoint.
    """
    logger = None
    config = None
    soundsystem = None
    last_schedule = None


    def __init__(self, config, soundsystem):
        """
        Initialize.
        """
        self.logger = logging.getLogger("AuraEngine")
        self.config = config
        self.soundsystem = soundsystem


    def on_play(self, entry):
        """
        Some track started playing.
        """
        self.store_trackservice(entry)
        self.store_show_info(entry)


    def store_trackservice(self, entry):
        """
        Posts the given `PlaylistEntry` to the Engine API Playlog.
        """
        data = dict()
        data["track_start"] = entry.entry_start
        data["track_artist"] = entry.meta_data.artist
        data["track_album"] = entry.meta_data.album
        data["track_title"] = entry.meta_data.title
        data["track_duration"] = entry.duration
        data["track_type"] = entry.get_type().numeric
        data["timeslot_id"] = entry.playlist.schedule.schedule_id
        data["show_name"] = entry.playlist.schedule.show_name
        data["log_source"] = self.config.get("api_engine_number")
        data = SU.clean_dictionary(data)

        self.logger.info("Posting schedule update to Engine API...")        
        url = self.config.get("api_engine_playlog_url")
        headers = {'content-type': 'application/json'}
        body = json.dumps(data, indent=4, sort_keys=True, default=str)
        response = requests.post(url, data=body, headers=headers)
        self.logger.info("Engine API response: %s" % response.status_code)



    def store_show_info(self, entry):
        """
        Posts the current and next show information to the Engine API.
        """
        current_schedule = entry.playlist.schedule
        if current_schedule == self.last_schedule:
            self.logger.info("Schedule didn't change since last update.")
        else:
            self.logger.info("Posting schedule update to Engine API...")
            # TODO Implement